import pickle 

data = {'schema-2': {'data' : 'some random dict'}}
schema_name = list(data.keys())[0]
schema_data = data[schema_name]

try:
    with open('try.barfi', 'rb') as handle_read:
        schemas = pickle.load(handle_read)
except FileNotFoundError:
    schemas = {}

print(schemas)

with open('try.barfi', 'wb') as handle_write:
    schemas[schema_name] = schema_data
    pickle.dump(schemas, handle_write, protocol=pickle.HIGHEST_PROTOCOL)

with open('try.barfi', 'rb') as handle_read:
    schemas = pickle.load(handle_read)
    print(schemas)


# st.write('### DAG Graph of the computational block-link')
# fig, ax = plt.subplots(figsize=(12, 6))
# nx.draw(_ce._graph, with_labels=True, node_color='lightblue',
#         node_size=500, labels=_ce._map_block_id_name)
# st.pyplot(fig)

# st.write(_compu_order)

# st.write(_ce._active_blocks)
# st.write(_ce._map_block_id_name)
# st.write(_ce._map_interface_id_block_id)
# st.write(_ce._map_interface_id_name)
# st.write(_ce._map_link_interface_id_from_to)
# st.write(_ce._map_link_interface_id_to_from)


import sys
sys.path.append('../')

from barfi import st_barfi, barfi_schemas, Block
import streamlit as st

feed = Block(name='Feed')
feed.add_output()
def feed_func(self):
    self.set_interface(name='Output 1', value=4)
feed.add_compute(feed_func)

splitter = Block(name='Splitter')
splitter.add_input()
splitter.add_output()
splitter.add_output()
def splitter_func(self):
    in_1 = self.get_interface(name='Input 1')
    value = (in_1/2)
    self.set_interface(name='Output 1', value=value)
    self.set_interface(name='Output 2', value=value)
splitter.add_compute(splitter_func)

mixer = Block(name='Mixer')
mixer.add_input()
mixer.add_input()
mixer.add_output()
def mixer_func(self):
    in_1 = self.get_interface(name='Input 1')
    in_2 = self.get_interface(name='Input 2')
    value = (in_1 + in_2)
    self.set_interface(name='Output 1', value=value)
mixer.add_compute(mixer_func)

result = Block(name='Result')
result.add_input()
def result_func(self):
    in_1 = self.get_interface(name='Input 1')
result.add_compute(result_func)

load_schema = st.selectbox('Select a saved schema:', barfi_schemas())

compute_engine = st.checkbox('Activate barfi compute engine', value=False)

if barfi_result := st_barfi(
    base_blocks=[feed, result, mixer, splitter],
    compute_engine=compute_engine,
    load_schema=load_schema,
):
    st.write(barfi_result)



from tkinter.tix import Select

from soupsieve import select
from barfi import Block

feed = Block(name='Feed')
feed.add_output()
def feed_func(self):
    self.set_interface(name='Output 1', value=4)
feed.add_compute(feed_func)

splitter = Block(name='Splitter')
splitter.add_input()
splitter.add_output()
splitter.add_output()
def splitter_func(self):
    in_1 = self.get_interface(name='Input 1')
    value = (in_1/2)
    self.set_interface(name='Output 1', value=value)
    self.set_interface(name='Output 2', value=value)
splitter.add_compute(splitter_func)

mixer = Block(name='Mixer')
mixer.add_input()
mixer.add_input()
mixer.add_output()
def mixer_func(self):    
    in_1 = self.get_interface(name='Input 1')
    in_2 = self.get_interface(name='Input 2')
    value = (in_1 + in_2)
    self.set_interface(name='Output 1', value=value)
mixer.add_compute(mixer_func)

result = Block(name='Result')
result.add_input()
def result_func(self):
    in_1 = self.get_interface(name='Input 1')
result.add_compute(result_func)

checkbox = Block(name='Checkbox')
checkbox.add_output()
checkbox.add_option(name='display-option', type='display', value='This is a Block with Checkbox option.')
checkbox.add_option(name='checkbox-option', type='checkbox')

input = Block(name='Input')
input.add_output()
input.add_option(name='display-option', type='display', value='This is a Block with Input option.')
input.add_option(name='input-option', type='input')

integer = Block(name='Integer')
integer.add_output()
integer.add_option(name='display-option', type='display', value='This is a Block with Integer option.')
integer.add_option(name='integer-option', type='integer')

number = Block(name='Number')
number.add_output()
number.add_option(name='display-option', type='display', value='This is a Block with Number option.')
number.add_option(name='number-option', type='number')

selecto = Block(name='Select')
selecto.add_output()
selecto.add_option(name='display-option', type='display', value='This is a Block with Select option.')
selecto.add_option(name='select-option', type='select', items=['Select A', 'Select B', 'Select C'])

slider = Block(name='Slider')
slider.add_output()
slider.add_option(name='display-option', type='display', value='This is a Block with Slider option.')
slider.add_option(name='slider-option', type='slider', min=0, max=10)

base_blocks = [feed, result, mixer, splitter, checkbox, input, integer, number, selecto, slider]
base_blocks_category = {'process': [feed, result, mixer, splitter], 'options':[checkbox, input, integer, number, selecto, slider]}



def build_option(name: str, type: str, kwargs):
    option = {'name': name}

    if type == 'checkbox':
        option['type'] = "CheckboxOption"
        value = kwargs.get('value', False)
        assert isinstance(
            value, bool), "Error: For checkbox option, 'value' must be of type boolean."
        option['value'] = value

    elif type == 'display':
        option['type'] = "TextOption"
        value = kwargs.get('value', "null display")
        assert isinstance(
            value, str), "Error: For display option, 'value' must be of type string."
        option['value'] = value

    elif type == 'input':
        option['type'] = "InputOption"
        value = kwargs.get('value', "Text input")
        assert isinstance(
            value, str), "Error: For input option, 'value' must be of type string."
        option['value'] = value

    elif type == 'integer':
        option['type'] = "IntegerOption"

        if 'value' in kwargs:
            value = kwargs.get('value')
            assert isinstance(
                value, int), "Error: For checkbox option, 'value' must be of type integer."
        else:
            value = None
        option['value'] = value

        if 'min' in kwargs:
            min = kwargs.get('min')
            assert isinstance(
                min, int), "Error: For checkbox option, 'min' must be of type integer."
        else:
            min = None
        option['min'] = min

        if 'max' in kwargs:
            max = kwargs.get('max')
            assert isinstance(
                max, int), "Error: For checkbox option, 'max' must be of type integer."
        else:
            max = None
        option['max'] = max
        option['properties'] = {'min': min, 'max': max}

    elif type == 'number':
        option['type'] = "NumberOption"

        if 'value' in kwargs:
            value = kwargs.get('value')
            assert isinstance(
                value, (int, float)
            ), "Error: For checkbox option, 'value' must be of type float or integer."
        else:
            value = None
        option['value'] = value

        if 'min' in kwargs:
            min = kwargs.get('min')
            assert isinstance(
                min, (int, float)
            ), "Error: For checkbox option, 'min' must be of type float or integer."
        else:
            min = None
        option['min'] = min

        if 'max' in kwargs:
            max = kwargs.get('max')
            assert isinstance(
                max, (int, float)
            ), "Error: For checkbox option, 'max' must be of type float or integer."
        else:
            max = None
        option['max'] = max
        option['properties'] = {'min': min, 'max': max}

    elif type == 'select':
        option['type'] = "SelectOption"

        if 'value' in kwargs:
            value = kwargs.get('value')
            assert isinstance(
                value, str), "Error: For select option, 'value' must be of string."
        else:
            value = None
        option['value'] = value

        items = kwargs.get('items', [])
        assert bool(
            items), "Error: There are no items specified for the select option."
        assert all(isinstance(item, str)
                    for item in items), "Error: items specified for the select option must all be of type string."
        option['items'] = items
        option['properties'] = {'items': items}

    elif type == 'slider':
        option['type'] = "SliderOption"

        if 'value' in kwargs:
            value = kwargs.get('value')
            assert isinstance(
                value, (int, float)
            ), "Error: For slider option, 'value' must be of type float or integer."
        else:
            value = None
        option['value'] = value

        min = kwargs.get('min', None)

        assert (
            min is not None), "Error: For slider option, 'min' must be specified."
        assert isinstance(
            min, (int, float)
        ), "Error: For slider option, 'min' must be of type float or integer."
        option['min'] = min

        max = kwargs.get('max', None)
        assert bool(
            max), "Error: For slider option, 'max' must be specified."
        assert isinstance(
            max, (int, float)
        ), "Error: For slider option, 'max' must be of type float or integer."
        option['max'] = max
        option['properties'] = {'min': min, 'max': max}

    else:
        raise('Error: No valid option type passed to the addOption method.')

    return option



from abc import ABCMeta, abstractmethod
from typing import Dict
    
class AbsSchemaManager(metaclass=ABCMeta):
    @abstractmethod
    def load_schemas(self, user: str) -> Dict:
        pass
    
    @abstractmethod
    def save_schema(self, schema_name: str, user: str, schema_data: Dict) -> bool:
        pass
      
    @abstractmethod
    def load_schema_name(self, schema_name: str, user: str) -> Dict:
        pass
    
    @abstractmethod
    def delete_schema(self, schema_name: str, user: str) -> bool:
        pass


import pickle
from typing import Dict
from barfi.schema.schema_manager import AbsSchemaManager

# Schema Manager
class LocalSchemaManager(AbsSchemaManager):
    def __init__(self, schema_path = None) -> None:
        if schema_path is None:
            schema_path = 'schemas.barfi'
        self.schema_path = schema_path
        
    
    def get_schema_name(self, schema_name: str, user: str) -> str:
        if user == '' or user is None:
            return schema_name
        return f'{schema_name}@{user}'
    
    
    def get_user(self, schema_name: str) -> str:
        if '@' in schema_name:
            return schema_name.split('@')[1]
        return ''
    
    def get_prefix_name(self, schema_name: str) -> str:
        if '@' in schema_name:
            return schema_name.split('@')[0]
        return schema_name
    
    def load_schemas(self, user: str) -> Dict:
        try:
            with open('schemas.barfi', 'rb') as handle_read:
                schemas = pickle.load(handle_read)
        except FileNotFoundError:
            schemas = {}

        schema_names = list(schemas.keys())
        
        if (len(schemas) == 0):
            return {'schema_names': schema_names, 'schemas': schemas}
        # Dict 按名称过滤
        name = self.get_schema_name('', user)
        if (name != '' and name is not None):
            _filtered = {key: value for key, value in schemas.items() if key.endswith(name)}
        else:
            _filtered = schemas
        keys = list(_filtered.keys())
        show_names = [self.get_prefix_name(key) for key in keys]
        return {'schema_names': show_names, 'schemas': _filtered}


    def save_schema(self, schema_name: str, user: str, schema_data: Dict) -> bool:
        try:
            with open('schemas.barfi', 'rb') as handle_read:
                schemas = pickle.load(handle_read)
        except FileNotFoundError:
            schemas = {}

        name = self.get_schema_name(schema_name, user)
        with open('schemas.barfi', 'wb') as handle_write:
            schemas[name] = schema_data
            pickle.dump(schemas, handle_write, protocol=pickle.HIGHEST_PROTOCOL)
        return True


    def load_schema_name(self, schema_name: str, user: str) -> Dict:
        schemas_barfi = self.load_schemas(user)
        name = self.get_schema_name(schema_name, user)
        if schema_name in schemas_barfi['schema_names']:
            return schemas_barfi['schemas'][name]
        else:
            raise ValueError(
                f'Schema :{schema_name}: not found in the saved schemas')


    def delete_schema(self, schema_name: str, user: str) -> Dict:
        try:
            with open('schemas.barfi', 'rb') as handle_read:
                schemas = pickle.load(handle_read)
        except FileNotFoundError:
            schemas = {}
        temp = {}
        del_schema_name = self.get_schema_name(schema_name, user)
        if del_schema_name in schemas:
            temp = schemas[del_schema_name]
            del schemas[del_schema_name]
        else:
            raise ValueError(
                f'Schema :{del_schema_name}: not found in the saved schemas')
        
        with open('schemas.barfi', 'wb') as handle_write:
            pickle.dump(schemas, handle_write, protocol=pickle.HIGHEST_PROTOCOL)
        return temp
    


import streamlit.components.v1 as components
from typing import List, Dict, Union

# import barfi components
from .block_builder import Block
from .compute_engine import ComputeEngine
from .schema.local_schema_manager import LocalSchemaManager
from .schema.schema_manager import AbsSchemaManager
from .schema.preset import editor_preset

import os

_RELEASE = True

# Declare a Streamlit component. `declare_component` returns a function
# that is used to create instances of the component. We're naming this
# function "_component_func", with an underscore prefix, because we don't want
# to expose it directly to users. Instead, we will create a custom wrapper
# function, below, that will serve as our component's public API.

# It's worth noting that this call to `declare_component` is the
# *only thing* you need to do to create the binding between Streamlit and
# your component frontend. Everything else we do in this file is simply a
# best practice.

if not _RELEASE:
    _component_func = components.declare_component(
        "st_barfi",
        url="http://localhost:3001",
    )
else:
    parent_dir = os.path.dirname(os.path.abspath(__file__))
    build_dir = os.path.join(parent_dir, "client")
    _component_func = components.declare_component(
        "st_barfi", path=build_dir)

# Create a wrapper function for the component. This is an optional
# best practice - we could simply expose the component function returned by
# `declare_component` and call it done. The wrapper allows us to customize
# our component's API: we can pre-process its input args, post-process its
# output value, and add a docstring for users.

# We use the special "key" argument to assign a fixed identity to this
# component instance. By default, when a component's arguments change,
# it is considered a new instance and will be re-mounted on the frontend
# and lose its current state. In this case, we want to vary the component's
# "name" argument without having it get recreated.

def st_barfi(base_blocks: Union[List[Block], Dict], 
             schemaManager: AbsSchemaManager = None, 
             load_schema: str = None, 
             compute_engine: bool = True, 
             key = None, 
             user: str = None, 
             show_menu: bool = True):

    if (schemaManager is not None) and (not isinstance(schemaManager, AbsSchemaManager)):
        raise TypeError(
            'Invalid type for schemaManager passed to the st_barfi component.')
    if (schemaManager is None): 
        schemaManager = LocalSchemaManager()

    editor_schema = schemaManager.load_schema_name(load_schema, user) if load_schema else None
    schemas_in_db = schemaManager.load_schemas(user)
    
    schema_names_in_db = schemas_in_db['schema_names']

    editor_setting = {'compute_engine': compute_engine}

    # base_blocks_data = [block._export() for block in base_blocks]

    if isinstance(base_blocks, List):
        base_blocks_data = [block._export() for block in base_blocks]
        base_blocks_list = base_blocks
    elif isinstance(base_blocks, Dict):
        base_blocks_data = []
        base_blocks_list = []
        for category, block_list in base_blocks.items():
            if not isinstance(block_list, List):
                raise TypeError(
                    'Invalid type for base_blocks passed to the st_barfi component.')
            for block in block_list:
                base_blocks_list.append(block)
                block_data = block._export()
                block_data['category'] = category
                base_blocks_data.append(block_data)
    else:
        raise TypeError('Invalid type for base_blocks passed to the st_barfi component.')
    _from_client = _component_func(base_blocks=base_blocks_data, load_editor_schema=editor_schema,
                                   load_schema_names=schema_names_in_db, load_schema_name=load_schema, show_menu=show_menu, editor_setting=editor_setting,
                                   key=key, default={'command': 'skip', 'editor_state': {}})

    if _from_client['command'] == 'flush':
        return _from_client
        
    if _from_client['command'] == 'execute':
        _ce = ComputeEngine(blocks=base_blocks_list)
        _ce.add_editor_state(_from_client['editor_state'])
        _ce._map_block_link()
        if not compute_engine:
            return _from_client
        _ce._execute_compute()
        return _ce.get_result()
    if _from_client['command'] == 'save':
        schemaManager.save_schema(schema_name=_from_client['schema_name'], schema_data=_from_client['editor_state'], user=user)
            
            
    if _from_client['command'] == 'load':
        load_schema = _from_client['schema_name']
        editor_schema = schemaManager.load_schema_name(load_schema, user)
    return {}


def barfi_schemas(schemaManager: AbsSchemaManager = None, user = None):
    if (schemaManager is not None) and (not isinstance(schemaManager, AbsSchemaManager)):
        raise TypeError(
            'Invalid type for schemaManager passed to the barfi_schemas function.')
    if (schemaManager is None):
        schemaManager = LocalSchemaManager()
        
    return schemaManager.load_schemas(user)['schema_names']



import sys
sys.path.append('../')
from matplotlib import pyplot as plt
from barfi import st_barfi, barfi_schemas
import streamlit as st
from test_blocks import base_blocks, base_blocks_category



barfi_schema_name = st.selectbox(
    'Select a saved schema to load:', barfi_schemas())

compute_engine = st.checkbox('Activate barfi compute engine', value=False)

if barfi_result := st_barfi(
    base_blocks=base_blocks_category,
    compute_engine=compute_engine,
    load_schema=barfi_schema_name,
):
    st.write(barfi_result)
    st.write(barfi_result['Feed-1']['block'].get_interface('Output 1'))
    st.write(barfi_result['Feed-2']['block'].get_interface('Output 1'))
    st.write(barfi_result['Splitter-1']['block'].get_interface('Input 1'))
    st.write(barfi_result['Splitter-1']['block'].get_interface('Output 1'))
    st.write(barfi_result['Splitter-1']['block'].get_interface('Output 2'))
    st.write(barfi_result['Mixer-1']['block'].get_interface('Input 1'))
    st.write(barfi_result['Mixer-1']['block'].get_interface('Input 2'))
    st.write(barfi_result['Mixer-1']['block'].get_interface('Output 1'))
    st.write(barfi_result['Result-1']['block'].get_interface('Input 1'))

    st.write(barfi_result['Feed-1']['block'].get_state('info'))
    st.write(barfi_result['Feed-2']['block'].get_state('info'))
    st.write(barfi_result['Splitter-1']['block'].get_state('info'))
    st.write(barfi_result['Mixer-1']['block'].get_state('info'))
    st.write(barfi_result['Result-1']['block'].get_state('info'))


import setuptools

long_description = """# Barfi

**Block and Link based graphical programming environment for performing simulations in Python.**

It provides a Streamlit component for a graphical programming environment to perform simulations.

Forked from Krish-adi's Barfi.
"""

setuptools.setup(
    name="yoyobarfi",
    version="0.8.1",
    author="YoYoDream",
    author_email="yoyodream729@gmail.com",
    description="Framework for a graphical programming environment. Forked from Krish-adi's Barfi.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yoyodream/barfi",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[
        "streamlit >= 0.63",
        "networkx >= 2.6.2",
    ],
    project_urls={
        'Source': 'https://github.com/yoyodream/barfi',
    },
)



import networkx as nx
import copy
from .block_builder import Block
from typing import List, Dict

class CyclicConnectionError(RecursionError, NotImplementedError):
    pass

class ComputeEngine(object):
    def __init__(self, blocks: List[Block] = [], **kwargs) -> None:
        # Initialise the Compute Enginge
        # The primitive block types that are tied to this compute engine
        self._blocks = blocks

        # The state of the editor that is stored as JSON/Dict that represents
        # the active blocks and links on the editor
        self._editor_state = {}

        # Active blocks extracted from the editor_state.
        self._active_blocks = {}

        # Mapping all the active blocks and connections to their id and names
        self._map_block_id_name = {}
        self._map_interface_id_block_id = {}
        self._map_interface_id_name = {}
        self._map_link_interface_id_from_to = {}
        self._map_link_interface_id_to_from = {}

        # Create a directed-acyclic-graph of the blocks and links
        # The computational graph
        self._graph = nx.DiGraph()

        # Result from the execution for the compute engine
        self._result = {}

    def add_editor_state(self, editor_state):
        self._editor_state = editor_state

    def get_result(self) -> Dict:
        # return self._active_blocks
        return self._result

    def _map_block_link(self):
        if not bool(self._editor_state):
            return
        for _block in self._editor_state['nodes']:
            # Create a child block object for the active block and associate with its id
            _parent_block = next(
                _b for _b in self._blocks if _b._type == _block['type'])
            # Create an independent deep copy of the parent block type
            _child_block = copy.deepcopy(_parent_block)
            _child_block._name = _block['name']

            # Map the active block id to its name
            self._map_block_id_name[_block['id']] = _block['name']

            # Map the active interfaces/Links to the active block
            _block_interfaces = {}
            for _interface in _block['interfaces']:
                self._map_interface_id_block_id[_interface[1]
                                                ['id']] = _block['id']
                self._map_interface_id_name[_interface[1]
                                            ['id']] = _interface[0]
                _child_block._set_interface_id(
                    name=_interface[0], id=_interface[1]['id'])
                _block_interfaces[_interface[0]] = _interface[1]

            for _option in _block['options']:
                _child_block.set_option(name=_option[0], value=_option[1])

            # Active blocks build from the base-blocks and editor-state
            self._active_blocks[_block['id']] = {
                'block': _child_block, 'interfaces': _block_interfaces, 'type': _block['type'], 'name': _block['name']}

            # Add block to the DAG as a node
            self._graph.add_node(_block['id'], name=_block['name'])

        for _connection in self._editor_state['connections']:
            from_node = self._map_interface_id_block_id[_connection['from']]
            to_node = self._map_interface_id_block_id[_connection['to']]
            if _connection['from'] not in self._map_link_interface_id_from_to:
                self._map_link_interface_id_from_to[_connection['from']] = [
                ]
            self._map_link_interface_id_from_to[_connection['from']].append(
                _connection['to'])
            self._map_link_interface_id_to_from[_connection['to']
                                                ] = _connection['from']

            # Add the _connection to the DAG as an edge
            self._graph.add_edge(from_node, to_node,
                                 edge_id=_connection['id'])

        if not nx.is_directed_acyclic_graph(self._graph):
            raise CyclicConnectionError('Cycle(s) detected. Not supported by Barfi at the moment.')
        else:
            _compu_order = [self._map_block_id_name[node]
                            for node in nx.topological_sort(self._graph)]

        # TODO morph _result to _active_blocks and have only one of them.
        # TODO add the interface info to the block
        for block_id, block in self._active_blocks.items():
            self._result[block['name']] = {'block': block['block'],
                                           'type': block['type'],
                                           'interfaces': {}}
            for link_id, link in block['interfaces'].items():
                self._result[block['name']]['interfaces'][link_id] = {}
                _interface_id = link['id']

                if _interface_id in self._map_link_interface_id_from_to:
                    self._result[block['name']
                                 ]['interfaces'][link_id]['type'] = 'output'
                    self._result[block['name']
                                 ]['interfaces'][link_id]['to'] = {}
                    for to_id in self._map_link_interface_id_from_to[_interface_id]:
                        to_name = self._map_interface_id_name[to_id]
                        to_block_id = self._map_interface_id_block_id[to_id]
                        to_block_name = self._map_block_id_name[to_block_id]
                        self._result[block['name']
                                     ]['interfaces'][link_id]['to'][to_block_name] = to_name

                if _interface_id in self._map_link_interface_id_to_from:
                    self._result[block['name']
                                 ]['interfaces'][link_id]['type'] = 'input'
                    self._result[block['name']
                                 ]['interfaces'][link_id]['from'] = {}
                    from_id = self._map_link_interface_id_to_from[_interface_id]
                    from_name = self._map_interface_id_name[from_id]
                    from_block_id = self._map_interface_id_block_id[from_id]
                    from_block_name = self._map_block_id_name[from_block_id]
                    self._result[block['name']
                                 ]['interfaces'][link_id]['from'][from_block_name] = from_name

    def _execute_compute(self):
        if not bool(self._editor_state):
            return
        skip_node = set()
        descendant_set = {}
        for node in nx.topological_sort(self._graph):
            if node not in skip_node:
                try:
                    self._active_blocks[node]['block']._on_compute()
                    self._active_blocks[node]['block']._state['info'] = {
                        'status': 'Computed'}

                    for _, value in self._active_blocks[node]['block']._outputs.items():
                        try:
                            for find_to in self._map_link_interface_id_from_to[value['id']]:
                                find_to_block = self._map_interface_id_block_id[find_to]
                                self._active_blocks[find_to_block]['block'].set_interface(
                                    name=self._map_interface_id_name[find_to], value=value['value'])
                        except:
                            pass

                except Exception as e:
                    self._active_blocks[node]['block']._state['info'] = {
                        'status': 'Errored', 'exception': e.args}
                    node_desc = nx.descendants(self._graph, node)
                    descendant_set[self._active_blocks[node]
                                   ['name']] = node_desc
                    skip_node = skip_node.union(node_desc)

            else:
                for parent, child_set in descendant_set.items():
                    if node in child_set:
                        break
                self._active_blocks[node]['block']._state['info'] = {
                    'status': 'Errored', 'message': 'Parent block errored', 'parent': parent}
